#include "ETpuLib.h"

// Data imported from other files
int g_failFlag;

// Functions local to this file, not to be exported.
void InternalErrorLoop(void);
void SimBackDoor( void *);
U32 *GetChanCfgRegAddr(int l_chan);
U32 *GetChanStatusRegAddr(int l_chan);
void SetBackdoorScriptHeaderAndDo(char *l_scriptName );

// Constants local to this file, not be exported
#define ETPU_DATA_BASE_ADDR ETPU_BASE_ADDR + 0x8000

#define DATA_TO_DATA24_MIRROR_OFFSETT 0x4000

U8 g_backDoorBuf[128];

//*****************************************************************************
//     Module Configuration Register Functions
//*****************************************************************************
void write_global_time_base_enable(int l_val)
{
	U32 *l_moduleCfgRegAddr = (U32*)(ETPU_BASE_ADDR + 0x0);
	U32 l_moduleCfgRegVal = *l_moduleCfgRegAddr;
	if( l_val != (l_val & 0x1) )
		InternalErrorLoop();
	l_moduleCfgRegVal &= (~0x00000001);
	l_moduleCfgRegVal |= l_val;
	*l_moduleCfgRegAddr = l_moduleCfgRegVal;
}

//*****************************************************************************
//     Time Base Configuration Register Functions
//*****************************************************************************
void write_entry_table_base_addr(U32 l_entryTableBaseAddr)
{
	U32 *l_engineCfgAddr = (U32*) (0x14 + ETPU_BASE_ADDR);
	U32 l_engineCfgReg = *l_engineCfgAddr;
	l_engineCfgReg &= ~0x1f;
	l_engineCfgReg |= (l_entryTableBaseAddr>>11) & 0x1f;
	*l_engineCfgAddr = l_engineCfgReg;
}

//*****************************************************************************
//     Time Base Configuration Register Functions
//*****************************************************************************
void write_tcr1_control(int l_val)
{
	U32 *l_timeBaseCfgRegAddr = (U32*)(ETPU_BASE_ADDR + 0x20);
	U32 l_timeBaseCfgRegVal = *l_timeBaseCfgRegAddr;
	if( l_val != (l_val & 3) )
		InternalErrorLoop();
	l_timeBaseCfgRegVal &= (~0x0000C000);
	l_timeBaseCfgRegVal |= (l_val<<14);
	*l_timeBaseCfgRegAddr = l_timeBaseCfgRegVal;
}

void write_tcr1_prescaler(int l_val)
{
	U32 *l_timeBaseCfgRegAddr = (U32*)(ETPU_BASE_ADDR + 0x20);
	U32 l_timeBaseCfgRegVal = *l_timeBaseCfgRegAddr;
	l_val--;
	if( l_val != (l_val & 0xff) )
		InternalErrorLoop();
	l_timeBaseCfgRegVal &= (~0x000000ff);
	l_timeBaseCfgRegVal |= (l_val<<0);
	*l_timeBaseCfgRegAddr = l_timeBaseCfgRegVal;
}

void write_tcr2_control(int l_val)
{
	U32 *l_timeBaseCfgRegAddr = (U32*)(ETPU_BASE_ADDR + 0x20);
	U32 l_timeBaseCfgRegVal = *l_timeBaseCfgRegAddr;
	if( l_val != (l_val & 0x7) )
		InternalErrorLoop();
	l_timeBaseCfgRegVal &= (~0xE0000000);
	l_timeBaseCfgRegVal |= (l_val<<29);
	*l_timeBaseCfgRegAddr = l_timeBaseCfgRegVal;
}

void write_tcr2_prescaler(int l_val)
{
	U32 *l_timeBaseCfgRegAddr = (U32*)(ETPU_BASE_ADDR + 0x20);
	U32 l_timeBaseCfgRegVal = *l_timeBaseCfgRegAddr;
	if( l_val != (l_val & 0x3F) )
		InternalErrorLoop();
	l_timeBaseCfgRegVal &= (~0x003F0000);
	l_timeBaseCfgRegVal |= (l_val<<16);
	*l_timeBaseCfgRegAddr = l_timeBaseCfgRegVal;
}

void write_tcrclk_filter(int l_val)
{
	U32 *l_timeBaseCfgRegAddr = (U32*)(ETPU_BASE_ADDR + 0x20);
	U32 l_timeBaseCfgRegVal = *l_timeBaseCfgRegAddr;
	if( l_val != (l_val & 0x3) )
		InternalErrorLoop();
	l_timeBaseCfgRegVal &= (~0x18000000);
	l_timeBaseCfgRegVal |= (l_val<<27);
	*l_timeBaseCfgRegAddr = l_timeBaseCfgRegVal;
}

void write_angle_mode(int l_val)
{
	U32 *l_timeBaseCfgRegAddr = (U32*)(ETPU_BASE_ADDR + 0x20);
	U32 l_timeBaseCfgRegVal = *l_timeBaseCfgRegAddr;
	if( l_val != (l_val & 0x1) )
		InternalErrorLoop();
	l_timeBaseCfgRegVal &= (~0x02000000);
	l_timeBaseCfgRegVal |= (l_val<<25);
	*l_timeBaseCfgRegAddr = l_timeBaseCfgRegVal;
}

//*****************************************************************************
//     Channel Configuration Register Functions
//*****************************************************************************
void write_chan_base_addr (int l_chan, int l_chanBaseAddr)
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U32 l_chanCfgRegVal = *l_chanCfgRegAddr;
	l_chanCfgRegVal &= (~0x7ff);
	l_chanCfgRegVal |= ((l_chanBaseAddr >> 3) & 0x7ff);
	*l_chanCfgRegAddr = l_chanCfgRegVal;
}

void write_chan_func (int l_chan, int l_func)
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U32 l_chanCfgRegVal = *l_chanCfgRegAddr;
	if( l_func != (l_func & 0x1f) )
		InternalErrorLoop();
	l_chanCfgRegVal &= (~0x001F0000);
	l_chanCfgRegVal |= (l_func << 16);
	*l_chanCfgRegAddr = l_chanCfgRegVal;
}

void write_chan_entry_condition (int l_chan, int l_entryCond)
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U32 l_chanCfgRegVal = *l_chanCfgRegAddr;
	if( l_entryCond != (l_entryCond & 0x1) )
		InternalErrorLoop();
	l_chanCfgRegVal &= (~0x01000000);
	l_chanCfgRegVal |= (l_entryCond << 24);
	*l_chanCfgRegAddr = l_chanCfgRegVal;
}

void write_chan_mode(int l_chan, int l_mode)
{
	U32 *l_chanStatusRegAddr = GetChanStatusRegAddr(l_chan);
	U32 l_chanStatusRegVal = *l_chanStatusRegAddr;
	if( l_mode != (l_mode & 0x3) )
		InternalErrorLoop();
	l_chanStatusRegVal &= (~3);
	l_chanStatusRegVal |= (l_mode);
	*l_chanStatusRegAddr = l_chanStatusRegVal;
}

void write_chan_cpr(int l_chan, int l_priority)
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U32 l_chanCfgRegVal = *l_chanCfgRegAddr;
	if( l_priority != (l_priority & 0x1f) )
		InternalErrorLoop();
	l_chanCfgRegVal &= (~0x30000000);
	l_chanCfgRegVal |= (l_priority << 28);
	*l_chanCfgRegAddr = l_chanCfgRegVal;
}

void write_chan_hsrr(int l_chan, int l_hostServiceRequest)
{
	U32 *l_chanHsrRegAddr = GetChanCfgRegAddr(l_chan)+2;
	if( l_hostServiceRequest != (l_hostServiceRequest & 0x7) )
		InternalErrorLoop();
	*l_chanHsrRegAddr = l_hostServiceRequest;
}

void write_chan_data32( int l_chan, U32 l_addrOffset, U32 l_val )
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U32 *l_chanDataAddr = (U32*) ( ETPU_DATA_BASE_ADDR +
						   ( ( (*l_chanCfgRegAddr) & 0x7FF) << 3) );
	if ( l_addrOffset >= 0x80 )
		InternalErrorLoop();
	l_chanDataAddr += (l_addrOffset>>2);
	*l_chanDataAddr = l_val;
}

void write_chan_data24( int l_chan, U32 l_addrOffset, U32 l_val )
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U32 *l_chanDataAddr = (U32*) ( ETPU_DATA_BASE_ADDR +
						   ( ( (*l_chanCfgRegAddr) & 0x7FF) << 3) );
	if ( l_addrOffset >= 0x80 )
		InternalErrorLoop();
	l_chanDataAddr += (l_addrOffset>>2);
	l_chanDataAddr += (DATA_TO_DATA24_MIRROR_OFFSETT>>2);
	*l_chanDataAddr = l_val;
}

void write_chan_data8( int l_chan, U32 l_addrOffset, U8 l_val )
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U8  *l_chanDataAddr = (U8*) ( ETPU_DATA_BASE_ADDR +
						   ( ( (*l_chanCfgRegAddr) & 0x7FF) << 3) );
	if ( l_addrOffset >= 0x80 )
		InternalErrorLoop();
	l_chanDataAddr += l_addrOffset;
	*l_chanDataAddr = l_val;
}

void verify_chan_data32( int l_chan, U32 l_addrOffset, U32 l_val )
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U32 *l_chanDataAddr = (U32*) ( ETPU_DATA_BASE_ADDR +
						   ( ( (*l_chanCfgRegAddr) & 0x7FF) << 3) );
	if ( l_addrOffset >= 0x80 )
		InternalErrorLoop();
	l_chanDataAddr += (l_addrOffset>>2);
	if( *l_chanDataAddr != l_val )
		g_failFlag++;
}

void verify_chan_data24( int l_chan, U32 l_addrOffset, U32 l_val )
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U32 *l_chanDataAddr = (U32*) ( ETPU_DATA_BASE_ADDR +
						   ( ( (*l_chanCfgRegAddr) & 0x7FF) << 3) );
	if ( l_addrOffset >= 0x80 )
		InternalErrorLoop();
	l_chanDataAddr += (l_addrOffset>>2);
	if( ((*l_chanDataAddr)&(0x00FFFFFF)) != (l_val&(0x00FFFFFF)) )
		g_failFlag++;
}

/*
void verify_chan_data8( int l_chan, U32 l_addrOffset, U8 l_val )
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U32 *l_chanDataAddr = (U32*) ( ETPU_DATA_BASE_ADDR +
						   ( ( (*l_chanCfgRegAddr) & 0x7FF) << 3) );
	if ( l_addrOffset >= 0x80 )
		InternalErrorLoop();
	l_chanDataAddr += (l_addrOffset>>2);
	if( ((*l_chanDataAddr)&(0x00FFFFFF)) != (l_val&(0x00FFFFFF)) )
		g_failFlag++;
}
*/

void enable_chan_intr(int l_chan)
{
	U32 *l_chanCfgRegAddr = GetChanCfgRegAddr(l_chan);
	U32 l_chanCfgRegVal = *l_chanCfgRegAddr;
	l_chanCfgRegVal |= (1<<31);
	*l_chanCfgRegAddr = l_chanCfgRegVal;
}

void verify_chan_intr(int l_chan, unsigned l_val)
{
	U32 *l_chanIntrStatusRegAddr = (U32*) (ETPU_BASE_ADDR + 0x200);
	U32 l_mask = 1<<l_chan;
	U32 l_actualVal = l_mask & (*l_chanIntrStatusRegAddr);
	U32 l_expectedVal = (l_val & 1) << l_chan;
	if( l_actualVal != l_expectedVal)
		g_failFlag++;
}

void clear_chan_intr(int l_chan)
{
	U32 *l_chanStatusRegAddr = GetChanCfgRegAddr(l_chan)+1;
	U32 l_mask = 1<<31;
	*l_chanStatusRegAddr = l_mask;
}

//*****************************************************************************
//     Pin Control
//*****************************************************************************
void write_chan_input_pin(int l_chan, int l_val)
{
	// Point the pointer at the "beaf"
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = l_chan;
	*l_u32Ptr++ = l_val;
	SetBackdoorScriptHeaderAndDo( "write_chan_input_pin" );
}

void verify_chan_output_pin(int l_chan, int l_val)
{
	// Point the pointer at the "beaf"
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = l_chan;
	*l_u32Ptr++ = l_val;
	SetBackdoorScriptHeaderAndDo( "verify_chan_output_pin" );
}

void write_tcrclk_pin( int l_val )
{
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = l_val;
	SetBackdoorScriptHeaderAndDo( "write_tcrclk_pin" );
}

//*****************************************************************************
//     Simulator control Functions
//*****************************************************************************
void set_clk_period(int l_femtoSeconds)
{
	// Point the pointer at the "beaf"
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = l_femtoSeconds;
	SetBackdoorScriptHeaderAndDo( "set_clk_period" );
}

//void wait_time(int l_microSeconds)
// Note that for some reason,
// these are being pushed as "double" in the introl compiler
void wait_time(double l_microSeconds)
{
	// Point the pointer at the "beaf"
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	U32 *l_microSecondsPtr = (U32 *) (&l_microSeconds);
	*l_u32Ptr++ = *l_microSecondsPtr++;
	*l_u32Ptr++ = *l_microSecondsPtr++;
	SetBackdoorScriptHeaderAndDo( "wait_time" );
}

void at_time(double l_microSeconds)
{
	// Point the pointer at the "beaf"
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	U32 *l_microSecondsPtr = (U32 *) (&l_microSeconds);
	*l_u32Ptr++ = *l_microSecondsPtr++;
	*l_u32Ptr++ = *l_microSecondsPtr++;
	SetBackdoorScriptHeaderAndDo( "at_time" );
}

void print_to_trace(char *l_str)
{
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = (U32) l_str;
	SetBackdoorScriptHeaderAndDo( "print_to_trace" );
}

void print(char *l_str)
{
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = (U32) l_str;
	SetBackdoorScriptHeaderAndDo( "print" );
}

void place_buffer(int l_from, int l_to)
{
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = l_from;
	*l_u32Ptr++ = l_to;
	SetBackdoorScriptHeaderAndDo( "place_buffer" );
}

void set_angle_indices(int l_degrees_per_cycle, int l_teeth_per_cycle)
{
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = l_degrees_per_cycle;
	*l_u32Ptr++ = l_teeth_per_cycle;
	SetBackdoorScriptHeaderAndDo( "set_angle_indices" );
}

//*****************************************************************************
//     Global Memory
//*****************************************************************************

void verify_mem_u8(enum ADDR_SPACE s, U32 address, U8  mask, U8  val)
{
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = s;
	*l_u32Ptr++ = address;
	*l_u32Ptr++ = mask;
	*l_u32Ptr++ = val;
	SetBackdoorScriptHeaderAndDo( "verify_mem_u8" );
}

void verify_mem_u16(enum ADDR_SPACE s, U32 address, U16 mask, U16 val)
{
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = s;
	*l_u32Ptr++ = address;
	*l_u32Ptr++ = mask;
	*l_u32Ptr++ = val;
	SetBackdoorScriptHeaderAndDo( "verify_mem_u16" );
}

void verify_mem_u32(enum ADDR_SPACE s, U32 address, U32 mask, U32 val)
{
	U32 *l_u32Ptr = (U32 *) (g_backDoorBuf + 0x10);
	*l_u32Ptr++ = s;
	*l_u32Ptr++ = address;
	*l_u32Ptr++ = mask;
	*l_u32Ptr++ = val;
	SetBackdoorScriptHeaderAndDo( "verify_mem_u32" );
}

//*****************************************************************************
//     Local Library functions
//*****************************************************************************

// Backdoor into the ASH WARE simulation environment
void SetBackdoorScriptHeaderAndDo(char *l_scriptName )
{
	// 128 Bytes Block for interfacing to the
	U32 *l_u32Ptr = (U32 *) g_backDoorBuf;

	// Save the first signatur
	*l_u32Ptr++ = 0xD59BF883;
	// Save the second signature
	*l_u32Ptr++ = 0xA9582945;
	// Save a NULL pointer, indicating to use default script target;
	*l_u32Ptr++ = 0;
	*l_u32Ptr++ = (U32) l_scriptName;

#ifndef WIN32
	SimBackDoor( g_backDoorBuf );
#endif
}

void InternalErrorLoop(void)
{
	while(1);
		g_failFlag=1;
}

U32 * GetChanCfgRegAddr(int l_chan)
{
	U32 *l_chanCfgRegAddr;
	if( l_chan >=0 && l_chan <=31)
		 l_chanCfgRegAddr = (U32*) (ETPU_BASE_ADDR + 0x400 + (l_chan * 16));
	else
		InternalErrorLoop();

	return l_chanCfgRegAddr;
}

U32 * GetChanStatusRegAddr(int l_chan)
{
	U32 *l_chanStatusRegAddr;
	if( l_chan >=0 && l_chan <=31)
		 l_chanStatusRegAddr = (U32*) (ETPU_BASE_ADDR + 0x404 + (l_chan * 16));
	else
		InternalErrorLoop();

	return l_chanStatusRegAddr;
}


