#ifndef __ETPU_LIBRARY_H__
#define __ETPU_LIBRARY_H__

enum ADDR_SPACE
{
	ETPU_CODE_SPACE=0x40,
	ETPU_CTRL_SPACE=0x08,
	ETPU_DATA_SPACE=0x20,
	ETPU_DATA_24_SPACE=0x10,
	ETPU_NODE_SPACE=0x80
};

// Define some data types
#define U32 unsigned long
#define U16 unsigned short
#define U8  unsigned char

// In some micro's, the eTpu is a dual-engine device.
enum ETPU_ENGINE_ID  { ETPU_ENGINE_A, ETPU_ENGINE_B };

#ifdef _MPC5554_TARGET_
#define  ETPU_BASE_ADDR  0xC3FC0000
#define  ETPU_PRAM_ADDR  (ETPU_BASE_ADDR +  0x8000)
#endif  // _MPC5554_TARGET_

// Functions that MATCH EXACTLY script commands
// and are therefore interchangeable with these script commands

// ETPU Module Configuration Register Functions   (ETPUMCR)
void write_global_time_base_enable(int val);

// Engine Configuration Register Functions  (ETPUECR)
void write_entry_table_base_addr(U32 addr);

// Time Base Configuration Register Functions  (ETPUTBCR)
void write_tcr1_control(int val);
void write_tcr1_prescaler(int val);
void write_tcr2_control(int val);
void write_tcr2_prescaler(int val);
void write_tcrclk_filter(int val);
void write_angle_mode(int val);

// Channel Configuration Registers Functions (CxCR, CxSCR, CxHSRR)
void write_chan_base_addr(int chan, int chanBaseAddr);
void write_chan_func(int chan, int func);
void write_chan_entry_condition(int chan, int condition);
void write_chan_mode(int chan, int mode);
void write_chan_cpr(int chan, int priority);
void write_chan_hsrr(int chan, int hostServiceRequest);

// Channel Interrupts
void enable_chan_intr(int chan);
void verify_chan_intr(int chan, unsigned val);
void clear_chan_intr(int chan);

// Channel Parameter RAM
void write_chan_data32( int chan, U32 addrOffset, U32 val );
void verify_chan_data32( int chan, U32 addrOffset, U32 val );
void write_chan_data24( int chan, U32 addrOffset, U32 val );
void verify_chan_data24( int chan, U32 addrOffset, U32 val );
void write_chan_data16( int chan, U32 addrOffset, U16 val );
void verify_chan_data16( int chan, U32 addrOffset, U16 val );
void write_chan_data8( int chan, U32 addrOffset, U8 val );
void verify_chan_data8( int chan, U32 addrOffset, U8 val );

// Memory
void verify_mem_u8 (enum ADDR_SPACE s, U32 address, U8  mask, U8  val);
void verify_mem_u16(enum ADDR_SPACE s, U32 address, U16 mask, U16 val);
void verify_mem_u32(enum ADDR_SPACE s, U32 address, U32 mask, U32 val);

// Pin Control
void write_chan_input_pin( int chan, int val);
void verify_chan_output_pin( int chan, int val);
void write_tcrclk_pin( int val );

// Simulator control Functions
void set_clk_period(int femtoSeconds);
// Note: The introl compiler seems to convert floats to doubles
//       no matter what on a function call.
//       To avoid problems, will simply make all calls, "doubles"        
void wait_time(double microSeconds);
void at_time(double microSeconds);
void print_to_trace(char *str);
void print(char *str);
void place_buffer(int from, int to);
void set_angle_indices(int degrees_per_cycle, int teeth_per_cycle);

#endif // __ETPU_LIBRARY_H__
