OUTPUT_ARCH(m68k)
OUTPUT_FORMAT(coff-m68k) /* not actually needed as this is the default anyways */

/* If special run-time support is needed; must then
 * run link via gcc (not ld) in order for it to find this library
 * without adding a search path.  The gcc archive also contains
 * software floating point routines if needed */
/*GROUP(-lgcc)*/

/*
 * The memory map look like this:
 * +--------------------+ 0x0	(no ROM monitor... start at 0)
 * | .vecs	vector table|
 * +--------------------+ 0x400
 * | .text              |
 * |        _etext      |
 * |        ctor list   | the ctor and dtor lists are for
 * |        dtor list   | C++ support only
 * +--------------------+
 * | .data              | initialized data goes here
 * |        _edata      |
 * +--------------------+
 * | .bss               |
 * |        __bss_start | start of bss, cleared by crt0
 * |        __bss_end   | start of heap, used by sbrk()
 * +--------------------+
 * .                    .
 * .                    .
 * .                    .
 * |        __stack     | top of stack
 * +--------------------+ 0x10000
 */

/*
 * The following memory setup describes the 
 * standard ASH WARE 68332 development board.
 */

MEMORY
{
  /* 128K total RAM, break it up to ease setup of the reset vector */
  ram1 (rwx) : ORIGIN = 0x0, LENGTH = 1K /* vector table; start at 0 - no ROM monitor */
  ram2 (rwx) : ORIGIN = 0x400, LENGTH = 127K
}

/*
 * allocate the stack to be at the top of memory, since the stack
 * grows down
 */

PROVIDE (__stack = 0x20000 - 8);

/*
 * map sections to memory
 */
SECTIONS
{
  .vecs : { vector.o } > ram1

  .text :
  {
    *(.text)

    /* This is only really neeed for C++ code */
    . = ALIGN(0x4);
     __CTOR_LIST__ = .;
    ___CTOR_LIST__ = .;
    LONG((__CTOR_END__ - __CTOR_LIST__) / 4 - 2)
    *(.ctors)
    LONG(0)
    __CTOR_END__ = .;
    __DTOR_LIST__ = .;
    ___DTOR_LIST__ = .;
    LONG((__DTOR_END__ - __DTOR_LIST__) / 4 - 2)
    *(.dtors)
     LONG(0)
    __DTOR_END__ = .;

    *(.rodata)
    *(.gcc_except_table) 
    __INIT_SECTION__ = . ;
    LONG (0x4e560000)	/* linkw %fp,#0 */
    *(.init)
    SHORT (0x4e5e)	/* unlk %fp */
    SHORT (0x4e75)	/* rts */

    __FINI_SECTION__ = . ;
    LONG (0x4e560000)	/* linkw %fp,#0 */
    *(.fini)
    SHORT (0x4e5e)	/* unlk %fp */
    SHORT (0x4e75)	/* rts */

    _etext = .;

    *(.lit)
  } > ram2 = 0x4e71

  .data :
  {
    *(.shdata)
    *(.data)
    _edata = .;
  } > ram2

  .bss :
  {
    . = ALIGN(0x4);
    __bss_start = . ;
    *(.shbss)
    *(.bss)
    *(COMMON)
    __bss_end =  ALIGN (0x8);
  } > ram2

  .stab 0 (NOLOAD) :
  {
    *(.stab)
  }

  .stabstr 0 (NOLOAD) :
  {
    *(.stabstr)
  }
}
