# GNU makefile for I2C test/demo simulation app

# variables to properly compile for target
MTDT_TARGET	= CPU32SIM
CPU_TARGET	= CPU32
CHIP_TARGET	= _MPC5554_TARGET_
OPT_LEVEL	= 0
# note: CHIP_TARGET is for sim support

# compile executables
# actually, all compilation phases can be done through gcc based upon file suffix
AS      = m68k-coff-as
CC      = m68k-coff-gcc
CXX     = m68k-coff-g++
LD      = m68k-coff-ld


# gcc cpu target name
CPU	= cpu32

# compile options: optimization level, output coff format, target cpu,
# enable warnings, pass in macros
IPATH = -I. -I.. -I../.. -I../mcu
CFLAGS = -c -O$(OPT_LEVEL) -gcoff -m$(CPU) -Wall -D$(MTDT_TARGET) -D$(CPU_TARGET) -D$(CHIP_TARGET) -DGNU_COMPILE $(IPATH)

# final link options: output a map file, include no standard library code
# if standard libraries are needed, use LDFLAGS_CC with -nostdlib
# removed and run via $(CC) instead of $(LD)
LDFLAGS_CC = -m$(CPU) -Wl,-Map=$(basename $@).map -nostartfiles
# final link options when run directly through $(LD)
LDFLAGS = -Map=$(basename $@).map


# define variables describing all the code to be compiled into the executable

# starup files
OBJECTS_STARTUP = \
	crt0_cpu32.o

# base object list
OBJECTS_BASE = \
	isrLib.o \
	ETpuLib.o

# app-specific objects
OBJECTS_APP = \
	etpu_util.o \
	etpu_init.o \
	etpu_i2c.o \
	etpu_i2c_master.o \
	etpu_i2c_slave.o

# objects for being able to run from reset (vector table)
OBJECTS_RESET = \
	vector.o

OBJECTS_ALL = $(OBJECTS_STARTUP) $(OBJECTS_BASE) $(OBJECTS_APP) $(OBJECTS_RESET)


# specify a search path for source files

VPATH = . \
	.. \
	../.. \
	../.. \
	../mcu

# default/main target - builds the final executable
# the link uses a command script as denoted by the -T option

all: I2C_sim.coff I2C_master_slave_sim.coff

I2C_sim.coff: $(OBJECTS_ALL) main.o
	$(CC) -Tlink_sys_sim.ld $(LDFLAGS_CC) -o $@ $(OBJECTS_ALL) main.o

I2C_master_slave_sim.coff: $(OBJECTS_ALL) main_master_slave.o
	$(CC) -Tlink_sys_sim.ld $(LDFLAGS_CC) -o $@ $(OBJECTS_ALL) main_master_slave.o


# compilation rules

# asm files that end in a little "s" are passed directly to the assembler
.s.o :
	$(CC) $(CFLAGS) $< -o $@
# asm files with a big "S" suffix get run through cpp (C preprocessor), 
# enabling the use of macros, include files, and other potentially nice features
.S.o :
	$(CC) $(CFLAGS) $< -o $@

.c.o :
	$(CC) $(CFLAGS) $< -o $@

.cpp.o :
	$(CXX) $(CFLAGS) $< -o $@


# example of generating dependencies
# run make -f [makefile] depend, then append text from depend.txt
# to this file below the "DEPENDENCY LISTS below" comment
depend:
	rm -f depend.txt
	$(CC) -M $(IPATH) crt0_cpu32.S >> depend.txt
	$(CC) -M $(IPATH) vector.S >> depend.txt
	$(CC) -M $(IPATH) main.c >> depend.txt
	$(CC) -M $(IPATH) isrLib.c >> depend.txt
	$(CC) -M $(IPATH) ETpuLib.c >> depend.txt
	$(CC) -M $(IPATH) etpu_init.c >> depend.txt
	$(CC) -M $(IPATH) etpu_util.c >> depend.txt
	$(CC) -M $(IPATH) etpu_i2c.c >> depend.txt
	$(CC) -M $(IPATH) etpu_i2c_master.c >> depend.txt
	$(CC) -M $(IPATH) etpu_i2c_slave.c >> depend.txt

# targets & rules to clean build output & force a complete recompile
clean:
	rm -f *.o *.map

cleanall:
	rm -f *.coff *.o *.map


# DEPENDENCY LISTS below

