/**************************************************************************
* FILE NAME: etpu_i2c.c
* 
* DESCRIPTION: Implementation of API for initializing and controlling the 
* I2C eTPU function.  See the .h file for API documentation.
*
*========================================================================
* REV      AUTHOR      DATE        DESCRIPTION OF CHANGE                 
* ---   -----------  ----------    ---------------------                 
* 1.0     J Diener   01/Dec/11     Initial version.     
*
* $Revision: 1.3 $
*
**************************************************************************/

#include "etpu_util.h"
#include "etpu_i2c.h"
#include "etpu_i2c_common.h"


// returned buffer ptr is in host data space (not eTPU-relative space)
int32_t aw_etpu_i2c_allocate_buffer(uint32_t byte_size,
									uint8_t** buf_ptr)
{
#ifdef ETPU_I2C_PARAMETER_CHECK
	if (!buf_ptr)
		return FS_ETPU_ERROR_VALUE;
#endif

	*buf_ptr = (uint8_t*)fs_etpu_malloc(byte_size);
	if (buf_ptr == 0)
		return (FS_ETPU_ERROR_MALLOC);
	return 0;
}


int32_t aw_etpu_i2c_shutdown(uint8_t channel)
{
	int32_t hsrr;
	volatile uint32_t cnt; // used to limit the shutdown wait
#ifdef ETPU_I2C_PARAMETER_CHECK
	if (((channel > (32 - ETPU_I2C_CHANNELS_USED)) && (channel < 64)) || (channel > 96 - (ETPU_I2C_CHANNELS_USED)))
		return FS_ETPU_ERROR_VALUE;
#endif

	hsrr = eTPU->CHAN[channel  ].HSRR.R + 
		   eTPU->CHAN[channel+1].HSRR.R +
		   eTPU->CHAN[channel+2].HSRR.R +
		   eTPU->CHAN[channel+3].HSRR.R;
	if (hsrr)
		return FS_ETPU_ERROR_NOT_READY;
	eTPU->CHAN[channel  ].HSRR.R = ETPU_I2C_SHUTDOWN_HSR;
	eTPU->CHAN[channel+1].HSRR.R = ETPU_I2C_SHUTDOWN_HSR;
	eTPU->CHAN[channel+2].HSRR.R = ETPU_I2C_SHUTDOWN_HSR;
	eTPU->CHAN[channel+3].HSRR.R = ETPU_I2C_SHUTDOWN_HSR;
	cnt = 0; while ((eTPU->CHAN[channel  ].HSRR.R != 0) && (cnt++ < 1000)) ; // wait for shutdown to occur
	cnt = 0; while ((eTPU->CHAN[channel+1].HSRR.R != 0) && (cnt++ < 1000)) ; // wait for shutdown to occur
	cnt = 0; while ((eTPU->CHAN[channel+2].HSRR.R != 0) && (cnt++ < 1000)) ; // wait for shutdown to occur
	cnt = 0; while ((eTPU->CHAN[channel+3].HSRR.R != 0) && (cnt++ < 1000)) ; // wait for shutdown to occur
	fs_etpu_disable( channel   );
	fs_etpu_disable( channel+1 );
	fs_etpu_disable( channel+2 );
	fs_etpu_disable( channel+3 );

	return 0;
}
