/**************************************************************************
* FILE NAME: etpu_init.c
* 
* DESCRIPTION: API implementation for initializing the eTPU module
*
*========================================================================
* REV      AUTHOR      DATE        DESCRIPTION OF CHANGE                 
* ---   -----------  ----------    ---------------------                 
* 1.0     J Diener   01/Dec/11     Initial version.     
*
* $Revision: 1.3 $
*
**************************************************************************/

#include "etpu_init.h"
#include "etpu_util.h"

// ALL OF THE BELOW INCLUDES SHOULD BE CAREFULLY CONTROLLED BY THE BUILD
// TO COME FROM THE CORRECT LOCATION, SO THAT THE SYSTEM IS INITIALIZED AS
// EXPECTED FOR THE GIVEN APP / SYSTEM BUILD

#include "etpu_set_defines.h"  /* API macros, auto-generated by ETEC */

#include "etpu_set_scm.c"  /* SCM code image, auto-generated by ETEC */

#include "etpu_set_idata.c"  /* initialized data (global, engine (eTPU2), channel frame), auto-generated by ETEC */

#include "etpu_config.h"  /* config specific for target & app (eTPU host interface register init structure */


/*------------------------------------------------------------------------------*/
uint32_t aw_etpu_init()
{
	uint32_t return_val;

	/* initialize eTPU hardware */
	return_val = fs_etpu_init (my_etpu_config, 
		(uint32_t *) _SCM_code_mem_array, sizeof (_SCM_code_mem_array),
		(uint32_t *) _global_mem_init, sizeof (_global_mem_init) );

	/* set the channel frame base address properly - channel frames will be allocated */
	/* starting at this address */
	/* this overrides the default init in fs_etpu_init(); in most cases the value will */
	/* be the same, except when engine RAM is used */
	fs_free_param = (uint32_t*)(fs_etpu_data_ram_start + _CHANNEL_FRAME_1ETPU_BASE_ADDR);

	return return_val;
}
